classdef ooGmsh4 < handle  
% CLASSDEF ooGmsh4:
%   
%
% Copyright (C) 2017-2020 F. Cuvelier
  properties
    q=[];
    nq=[];
    dim=[];
    d=[]; % d-simplicial mesh, 0 points, 1 curves, 2 surfaces, 3 volumes
    toGlobal=[];
    meshfile=[];
    partitionnedfile=false;
    MeshFormat=[];  
    PhysicalNames=[];
    Entities=[];
    PartitionedEntities=[];
    Nodes=[];
    Elements=[];
    PeriodicLinks=[];
    Info=[];
    orders=1;
  end
  
  methods
  
    function self=ooGmsh4(gmsh_file,varargin)
    % ooGmsh4 constructor, create an ooGmsh4 object from a gmsh mesh file.
    %   
    % USAGE:
    %   Gh=ooGmsh4(gmsh_file);
    %
    % INPUT:
    %   gmsh_file: name (string) of the gmsh mesh file to read.
    %
    % OUTPUT:
    %   Gh: an ooGmsh4 object.
    %
    % Copyright (C) 2017-2020 F. Cuvelier
      [fid,message]=fopen(gmsh_file,'r');
      if ( fid == -1 )
          error([message,' : ',gmsh_file]);
      end
      self.meshfile=gmsh_file;
      self.MeshFormat=fc_oogmsh.msh.read_MeshFormat(fid);
      assert( ismember(self.MeshFormat.version,{'4','4.1'}) , 'Needs version 4 or 4.1 of msh file format! Found version %s',self.MeshFormat.version )
      switch self.MeshFormat.version
        case '4.1'
          self.PhysicalNames=fc_oogmsh.msh4_1.read_PhysicalNames(fid,varargin{:});
          self.Entities=fc_oogmsh.msh4_1.read_Entities(fid,varargin{:});
          self.PartitionedEntities=fc_oogmsh.msh4_1.read_PartitionedEntities(fid,varargin{:});
          self.Nodes=fc_oogmsh.msh4_1.read_Nodes(fid,varargin{:});
          self.Elements=fc_oogmsh.msh4_1.read_Elements(fid,varargin{:});
          self.PeriodicLinks=fc_oogmsh.msh4_1.read_Periodic(fid,varargin{:});
        case '4'
          self.PhysicalNames=fc_oogmsh.msh4_0.read_PhysicalNames(fid,varargin{:});
          self.Entities=fc_oogmsh.msh4_0.read_Entities(fid,varargin{:});
          self.PartitionedEntities=fc_oogmsh.msh4_0.read_PartitionedEntities(fid,varargin{:});
          self.Nodes=fc_oogmsh.msh4_0.read_Nodes(fid,varargin{:});
          self.Elements=fc_oogmsh.msh4_0.read_Elements(fid,varargin{:});
          self.PeriodicLinks=fc_oogmsh.msh4_0.read_Periodic(fid,varargin{:});
      end
      fclose(fid);
      self.partitionnedfile=~isempty(self.PartitionedEntities);
      [self.q,self.toGlobal]=get_q(self);
      if sum(abs(self.q(3,:)))==0,self.q=self.q(1:2,:);end
      [self.dim,self.nq]=size(self.q);
      self.Info.meshfile=gmsh_file;
      self.set_d();
      self.set_points();
    end % ooGmsh4 constructor
    
    function set_d(self)
      if self.Entities.numVolumes>0, self.d=3;return;end
      if self.Entities.numSurfaces>0, self.d=2;return;end
      if self.Entities.numCurves>0, self.d=1;return;end
      if self.Entities.numPoints>0, self.d=0;return;end
      self.d=-1;
    end
    
    function set_points(self)
      if fc_tools.comp.isOctave()
        Point=@(q) q;
      else
        if  verLessThan('matlab','R2016b') % % repmat required when using Matlab < R2016b
          Point=@(q) repmat(q,1,self.nq);
        else
          Point=@(q) q;
        end
      end
      for i=1:self.Entities.numPoints
        self.Entities.Points(i).Point=self.Entities.Points(i).Point(1:self.dim);
        self.Entities.Points(i).nodeTag=find(sum(abs(self.q-Point(self.Entities.Points(i).Point)))==0); 
      end
    end
    
    function [q,tags]=get_q(self,varargin)
%      dimEntities=[self.Nodes.EntityBlocks(:).entityDim];
%      p = inputParser;
%      p.addParamValue('dimEntity',max(dimEntities),@(x) ismember(x,dimEntities));
%      p.parse(varargin{:});
%      dimEntity=p.Results.dimEntity;
%      idx=find(dimEntities==dimEntity);
      tags=horzcat(self.Nodes.EntityBlocks(:).nodeTags);
      q=horzcat(self.Nodes.EntityBlocks(:).Nodes);
    end
    
    function me=get_me(self,i)
      me=self.Elements.EntityBlocks(i).nodeTags;
    end
    
    function disp(self)
      fc_tools.comp.dispobject(self)
    end
    
    function info(self,varargin)
      fc_tools.comp.infoobject(self,varargin{:})
    end
    
    function tags=get_ElementaryTags(Gh,type)
      idx=find(Gh.Elements.ElementTypes==type);
      if isempty(idx), tags=[];return;end
      tags=[Gh.Elements.EntityBlocks(idx).entityTag];
    end
    
    function tags=get_ElementaryTags_entityDim(Gh,entityDim)
%        idx=find(Gh.Elements.ElementTypes==type);
%        if isempty(idx), tags=[];return;end
%        tags=[Gh.Elements.EntityBlocks(idx).entityTag];
      switch entityDim
        case 0 % Points
          tags=[Gh.Entities.Points(:).tag];
        case 1 % Curves
          tags=[Gh.Entities.Curves(:).tag];
        case 2 % Surfaces
          tags=[Gh.Entities.Surfaces(:).tag];
        case 3 % Volumes
          tags=[Gh.Entities.Volumes(:).tag];
      end
      
    end
    
    function tags=get_PhysicalTags_entityDim(Gh,entityDim)
      switch entityDim
        case 0 % Points
          tags=unique([Gh.Entities.Points(:).PhysicalTags]);
        case 1 % Curves
          tags=unique([Gh.Entities.Curves(:).PhysicalTags]);
        case 2 % Surfaces
          tags=unique([Gh.Entities.Surfaces(:).PhysicalTags]);
        case 3 % Volumes
          tags=unique([Gh.Entities.Volumes(:).PhysicalTags]);
      end
    end
    
    function tags=get_PartitionTags(Gh)
      tags=Gh.get_PartitionTags_entityDim(Gh.d);
%        tags=[];
%        if isempty(Gh.PartitionedEntities), return;end
%        if Gh.PartitionedEntities.numVolumes>0
%          tags=unique([Gh.PartitionedEntities.Volumes(:).partitionTags]);
%          return;
%        end  
%        if Gh.PartitionedEntities.numSurfaces>0
%          tags=unique([Gh.PartitionedEntities.Surfaces(:).partitionTags]);
%          return;
%        end 
    end
    
    function tags=get_PartitionTags_entityDim(Gh,entityDim)
      if ~Gh.partitionnedfile, tags=[];return;end % not partitioned!
      assert( nargin==2)
      %uniq=
      switch entityDim
        case 0 % Points
          tags=unique([Gh.PartitionedEntities.Points(:).partitionTags]);
        case 1 % Curves
          tags=unique([Gh.PartitionedEntities.Curves(:).partitionTags]);
        case 2 % Surfaces
          tags=unique([Gh.PartitionedEntities.Surfaces(:).partitionTags]);
        case 3 % Volumes
          tags=unique([Gh.PartitionedEntities.Volumes(:).partitionTags]);
      end
      
    end
    
    function [tags,boundtags]=get_PartitionTags_entityDimV2(Gh,entityDim)
      if ~Gh.partitionnedfile, tags=[];boundtags=[];return;end % not partitioned!
      assert( nargin==2)
      %uniq=
      switch entityDim
        case 0 % Points
          tags=unique([Gh.PartitionedEntities.Points(:).partitionTags]);
        case 1 % Curves
          tags=unique([Gh.PartitionedEntities.Curves(:).partitionTags]);
          boundtags={Gh.PartitionedEntities.Curves(:).partitionTags};
        case 2 % Surfaces
          tags=unique([Gh.PartitionedEntities.Surfaces(:).partitionTags]);
        case 3 % Volumes
          tags=unique([Gh.PartitionedEntities.Volumes(:).partitionTags]);
      end
      
    end
    
    function tags=get_BoundingTags_entityDim(Gh,entityDim)
      if ~Gh.partitionnedfile, tags=[];return;end % not partitioned!
      assert( nargin==2)
      %uniq=
      switch entityDim
        case 1 % Curves
          tags={Gh.PartitionedEntities.Curves(:).pointTags};
        case 2 % Surfaces
          tags={Gh.PartitionedEntities.Surfaces(:).curveTags};
        case 3 % Volumes
          tags={Gh.PartitionedEntities.Volumes(:).surfaceTags};
      end
    end
    
    function types=get_ElementTypes(Gh)
      types=unique(Gh.Elements.ElementTypes);
    end
    
    function sE=get_subEntities(Gh,EltType)
      if nargin==2
        sE=[];
        elt=fc_oogmsh.gmsh.elm_type_desc(EltType);
        assert( ~isempty(elt) )
        idx=find(Gh.Elements.ElementTypes==EltType);
        if isempty(idx),return;end
        EltsTags=[Gh.Elements.EntityBlocks(idx).entityTag];
      end
      entityDim=elt.d;
      switch entityDim
        case 0 % Points
          sE=Gh.Entities.Points;
        case 1 % Curves
          sE=Gh.Entities.Curves;
        case 2 % Surfaces
          sE=Gh.Entities.Surfaces;
        case 3 % Volumes
          sE=Gh.Entities.Volumes;
      end
      if nargin==2
        sEtags=[sE(:).tag]; 
        I=ismember(sEtags,EltsTags);
        sE=sE(I);
      end
    end
    
    function sE=get_subElementsDim(Gh,EntityDim)
      assert(ismember(EntityDim,0:3))
      sE=Gh.Elements.EntityBlocks([Gh.Elements.EntityBlocks(:).entityDim]==EntityDim);
    end
    
    function sE=get_subElementDimTag(Gh,EntityDim,tag)
        sE=get_subEntities(Gh);
%        tags=[sE.entityTag
%        if nargin==2
%          sE=[];
%          elt=fc_oogmsh.gmsh.elm_type_desc(EltType);
%          assert( ~isempty(elt) )
%          idx=find(Gh.Elements.ElementTypes==EltType);
%          if isempty(idx),return;end
%          EltsTags=[Gh.Elements.EntityBlocks(idx).entityTag];
%        end
%  
%        entityDim=elt.d;
%        switch entityDim
%          case 0 % Points
%            sE=Gh.Entities.Points;
%          case 1 % Curves
%            sE=Gh.Entities.Curves;
%          case 2 % Surfaces
%            sE=Gh.Entities.Surfaces;
%          case 3 % Volumes
%            sE=Gh.Entities.Volumes;
%        end
%        if nargin==2
%          sEtags=[sE(:).tag]; 
%          I=ismember(sEtags,EltsTags);
%          sE=sE(I);
%        end
    end

    function sE=get_subPartitionedEntities(Gh,entityDim)%,parttag)
      assert(ismember(entityDim,0:3))
      
      switch entityDim
        case 0 % Points
          sE=Gh.PartitionedEntities.Points;
        case 1 % Curves
          sE=Gh.PartitionedEntities.Curves;
        case 2 % Surfaces
          sE=Gh.PartitionedEntities.Surfaces;
        case 3 % Volumes
          sE=Gh.PartitionedEntities.Volumes;
      end
%        idx=cellfun(@(x) ismember(partnum,x),{sE(:).partitionTags})
%        sE=sE(idx);
    end
    
    function sE=get_Entities(Gh,entityDim)%,parttag)
      assert(ismember(entityDim,0:3))      
      switch entityDim
        case 0 % Points
          sE=Gh.Entities.Points;
        case 1 % Curves
          sE=Gh.Entities.Curves;
        case 2 % Surfaces
          sE=Gh.Entities.Surfaces;
        case 3 % Volumes
          sE=Gh.Entities.Volumes;
      end
    end
    
    function tags=get_PhysicalTags(Gh,EltType)
      sE=Gh.get_subEntities(EltType);
      if isempty(sE), tags=[];else, tags=unique([sE.PhysicalTags]);end
    end
    
    
    
    function sE=get_EntityTag(Gh,EntityDim,EntityTag)
      sE=Gh.get_Entities(EntityDim);
      tags=[sE.tag];
      idx=find(tags==EntityTag);
      if isempty(idx), sE=[];return;end
      assert(length(idx)==1)
      sE=sE(idx);
    end
    
    function tags=get_EntityTags_From_EntityBlocks(Gh,EntityDim)
      idx=([Gh.Elements.EntityBlocks(:).entityDim]==EntityDim);
      tags=[Gh.Elements.EntityBlocks(idx).entityTag];
    end
            
%      function tags=get_EntityTags_From_ParentTag(Gh,EntityDim,ParentTag)
%        % get tags (in Elements.EntityBlocks(:).entityTag given by a ParentTag ( oGh4p.PartitionedEntities.Surfaces(:).parentTag
%        idx=([Gh.Elements.EntityBlocks(:).entityDim]==EntityDim);
%        sE=Gh.Elements.EntityBlocks(idx);
%        sE=sE([sE(:).parentTag]==ParentTag);
%        tags=[sE(:).entityTag];
%      end
    
    function me=get_me_From_EntityBlock(Gh,EntityDim,EntityTag)
      me=[];
      %if EntityDim>0
        idx=([Gh.Elements.EntityBlocks(:).entityDim]==EntityDim) & ([Gh.Elements.EntityBlocks(:).entityTag]==EntityTag);
        if sum(idx)==0
          %keyboard
          if EntityDim==0
            idx1=find([Gh.Entities.Points(:).tag]==EntityTag);
            if length(idx1)==1
              me=Gh.Entities.Points(idx1).nodeTag;
              return;
            end
          end
          % could be center of a circle which not always in the nodes of the mesh
          %warning('Unable to find EntityBlock(EntityDim=%d,EntityTag=%d)\n',EntityDim,EntityTag)
          return;
        end
        assert(sum(idx)==1)
        me=Gh.Elements.EntityBlocks(idx).nodeTags;
      %else
        %me=EntityTag; %????
      %end
    end
    
    function varargout=get_localmesh_From_EntityBlock(self,EntityDim,EntityTag)
      % returns  [q,me] or [q,me,toGlobal]
      q=[];toGlobal=[];
      me=self.get_me_From_EntityBlock(EntityDim,EntityTag);
      if ~isempty(me)
        [q,me,toGlobal]=fc_oogmsh.msh.FromGlobalToLocal(self.q,me,self.toGlobal);
      else
        
      end
      varargout{1}=q;
      varargout{2}=me;
      if nargout==3,varargout{3}=toGlobal;end  
    end  
    
    function varargout=get_localmesh_EntityTag(self,EltType,EltTag,varargin)
      assert( ismember(nargout,[2,3]) )
      % returns  [q,me] or [q,me,toGlobal]
      p = inputParser;
      p.addParamValue('dim',3,@(x) ismember(x,[2,3]));
      p.parse(varargin{:});
      dim=p.Results.dim;
      me=self.get_me_ElementaryTag(EltType,EltTag);
      %[q,qtags]=self.get_q();
      [q,me,toGlobal]=fc_oogmsh.msh.FromGlobalToLocal(self.q,me,self.toGlobal);
      varargout{1}=q;
      varargout{2}=me;
      if nargout==3,varargout{3}=toGlobal;end  
    end    
    
    function varargout=get_localmesh_ElementaryTag(self,EltType,EltTag,varargin)
      assert( ismember(nargout,[2,3]) )
      % returns  [q,me] or [q,me,toGlobal]
      p = inputParser;
      p.addParamValue('dim',3,@(x) ismember(x,[2,3]));
      p.parse(varargin{:});
      dim=p.Results.dim;
      me=self.get_me_ElementaryTag(EltType,EltTag);
      %[q,qtags]=self.get_q();
      [q,me,toGlobal]=fc_oogmsh.msh.FromGlobalToLocal(self.q,me,self.toGlobal);
      varargout{1}=q;
      varargout{2}=me;
      if nargout==3,varargout{3}=toGlobal;end  
    end     
    
    function varargout=get_localmesh_PhysicalTag(self,EltType,PhysTag,varargin)
      assert( ismember(nargout,[2,3]) )
      % returns  [q,me] or [q,me,toGlobal]
      p = inputParser;
      p.addParamValue('dim',3,@(x) ismember(x,[2,3]));
      p.parse(varargin{:});
      dim=p.Results.dim;
      me=self.get_me_PhysicalTag(EltType,PhysTag);
      [q,me,toGlobal]=fc_oogmsh.msh.FromGlobalToLocal(self.q,me,self.toGlobal);
      varargout{1}=q;
      varargout{2}=me;
      if nargout==3,varargout{3}=toGlobal;end  
    end
    
%      function varargout=get_localmesh_PhysicalTag_dimEntity(self,dimEntity,PhysTag,varargin)
%        assert( ismember(nargout,[2,3]) )
%        % returns  [q,me] or [q,me,toGlobal]
%        p = inputParser;
%        p.addParamValue('dim',3,@(x) ismember(x,[2,3]));
%        p.parse(varargin{:});
%        dim=p.Results.dim;
%        me=self.get_me_PhysicalTag_dimEntity(dimEntity,PhysTag);
%        %[q,qtags]=self.get_q();
%        [q,me,toGlobal]=fc_oogmsh.msh.FromGlobalToLocal(self.q,me,self.toGlobal);
%  %        if dim==2
%  %          assert( sum(abs(q(3,:)))==0, 'Not a 2D mesh, cannot force dimension to 2')
%  %          q=q(1:2,:);
%  %        end
%        varargout{1}=q;
%        varargout{2}=me;
%        if nargout==3,varargout{3}=toGlobal;end  
%      end

    function me=get_me_ElementaryTag(Gh,type,tag) % tag is an ElementaryTag
      idx=find(Gh.Elements.ElementTypes==type);
      if isempty(idx), me=[];return;end
      i=find([Gh.Elements.EntityBlocks(idx).entityTag]==tag);
      if isempty(i), me=[];return;end
      assert(isscalar(i))
      me=Gh.Elements.EntityBlocks(idx(i)).nodeTags;
    end
    
%      function me=get_me_PhysicalTag_dimEntity(self,dimEntity,phtag)
%      % only works with order 1 simplices.
%        switch dimEntity 
%          case 1 % Curves
%            Entities=self.Entities.Curves;
%            elementtype=1; % line (see fc_oogmsh.gmsh.elm_type_list function)
%          case 2 % Surfaces
%            Entities=self.Entities.Surfaces;
%            elementtype=2; % triangle (see fc_oogmsh.gmsh.elm_type_list function)
%          case 3 % Volumes
%            Entities=self.Entities.Volumes;
%            elementtype=4; %(see fc_oogmsh.gmsh.elm_type_list function)
%        end
%        numPTs=[Entities(:).numPhysicalTags];
%        idx=find(numPTs>0);
%        Entities=Entities(idx);
%        %phtags=horzcat(Entities(:).PhysicalTags);
%        %idx=find(phtags==phtag);
%        phtags={Entities(:).PhysicalTags}; 
%        idx=cellfun(@(x) ismember(phtag,x) , phtags);
%        tags=horzcat(Entities(idx).tag); 
%        idx=find(self.Elements.ElementTypes==elementtype); % 1: Lines
%        Etags=[self.Elements.EntityBlocks(idx).entityTag];
%        [~,~,ib]=intersect(tags,Etags); % Etags(ib)==tags
%        Idx=idx(ib);
%        me=[self.Elements.EntityBlocks(Idx).nodeTags];
%      end
    
    function me=get_me_PhysicalTag(Gh,EltType,phtag)
      elt=fc_oogmsh.gmsh.elm_type_desc(EltType);
      idx=find(Gh.Elements.ElementTypes==EltType);
      sE=Gh.get_subEntities(EltType);
      J=cellfun(@(x) ismember(phtag,x),  {sE(:).PhysicalTags});
      tags=[sE(J).tag];
      i=ismember([Gh.Elements.EntityBlocks(idx).entityTag],tags);
      me=[Gh.Elements.EntityBlocks(idx(i)).nodeTags];
    end

    function me=get_me_PartitionPhysicalTag(Gh,EltType,parttag,phtag)
%        elt=fc_oogmsh.gmsh.elm_type_desc(EltType);
%        idx=find(Gh.Elements.ElementTypes==EltType);
%        sE=Gh.get_subEntities(EltType);
%        J=cellfun(@(x) ismember(phtag,x),  {sE(:).PhysicalTags});
%        tags=[sE(J).tag];
%        i=ismember([Gh.Elements.EntityBlocks(idx).entityTag],tags);
%        me=[Gh.Elements.EntityBlocks(idx(i)).nodeTags];
    end
    
    function tags=get_elementaryTags_in_PhysicalTag(self,entityDim,phTag)
      sE=self.get_Entities(entityDim);
      idx=cellfun(@(x) ismember(phTag,x),{sE(:).PhysicalTags});
      tags=[sE(idx).tag];
    end
    
    function tags=get_EntityTags_From_ElementaryTag(Gh,entityDim,elTag)
      % eltag : elementary tag or geometrical tag (set in .geo file)
      if ~isempty(Gh.PartitionedEntities)
        sE=get_subPartitionedEntities(Gh,entityDim);
        sE=sE([sE(:).numPartitions]==1); % on enlève les interfaces
        
      else
        sE=get_Entities(Gh,entityDim);
      end
      idx=cellfun(@(x) ismember(elTag,x),{sE(:).parentTag});
      tags=[sE(idx).tag];
    end
    
    function tags=get_EntityTags_From_parentTag(Gh,entityDim,pTag)
      if ~isempty(Gh.PartitionedEntities)
        sE=get_subPartitionedEntities(Gh,entityDim);
        sE=sE([sE(:).numPartitions]==1); % on enlève les interfaces
        idx=[sE(:).parentTag]==pTag;
        tags=[sE(idx).tag]; 
      else
        tags=pTag; 
      end
    end

    function tags=get_EntityTags_From_partitionTag(Gh,entityDim,pTag)
      if isempty(Gh.PartitionedEntities),tags=[];return;end
      sE=get_subPartitionedEntities(Gh,entityDim);
      idx=cellfun(@(x) ismember(pTag,x),{sE(:).partitionTags});
      tags=[sE(idx).tag];
    end
    
    function tags=get_EntityTags_From_PhysicalTag(Gh,entityDim,phTag)
      if ~isempty(Gh.PartitionedEntities)
        sE=get_subPartitionedEntities(Gh,entityDim);
        sE=sE([sE(:).numPartitions]==1); % on enlève les interfaces
        
      else
        sE=get_Entities(Gh,entityDim);
      end
      idx=cellfun(@(x) ismember(phTag,x),{sE(:).PhysicalTags});
      tags=[sE(idx).tag];
    end
    
    function find_PartitionInterfacesTags(Gh,entityDim)
      sE=get_subPartitionedEntities(Gh,entityDim);
      ST={Gh.PartitionedEntities.Volumes(:).surfaceTags}
    end
    
    function tags=get_InterfaceTags(Gh,entityDim) % entityDim of the interfaces
      bTags=Gh.get_BoundingTags_entityDim(entityDim+1) % negative tags are interfaces???
      bTags=cellfun(@(x) x',bTags,'UniformOutput',false);
      bT=[bTags{:}];
      tags=unique(-bT(bT<0)); % return absolute values of negative tags
    end
    
    function tags=get_InterfaceTags2(Gh,entityDim) % entityDim of the interfaces
      tags=[];
      if ~isempty(Gh.PartitionedEntities)
        sE=get_subPartitionedEntities(Gh,entityDim);
        sE=sE([sE(:).numPartitions]>1); % supposition numPartitions > 1 <=> interfaces ??? 
        tags=[sE(:).tag];
      end
    end
    
    function plotElementaryElementsDim(Gh,dim,varargin)
      fc_oogmsh.graphics.ooGmsh4.plotElementaryElementsDim(Gh,dim,varargin{:})
    end
    
    function plotElementaryElements(Gh,varargin)
      fc_oogmsh.graphics.ooGmsh4.plotElementaryElements(Gh,varargin{:})
    end
    
    function plotInterfaceElements(Gh,varargin)
      fc_oogmsh.graphics.ooGmsh4.plotInterfaceElements(Gh,varargin{:})
    end
    
    function plotPartitionElements(Gh,varargin)
      fc_oogmsh.graphics.ooGmsh4.plotPartitionElements(Gh,varargin{:})
    end
    
    function plotPhysicalElements(Gh,varargin)
      fc_oogmsh.graphics.ooGmsh4.plotPhysicalElements(Gh,varargin{:})
    end
    
    function plotmesh(Gh,d,varargin)
      fc_oogmsh.graphics.ooGmsh4.plotmesh(Gh,d,varargin{:})
    end
  end % methods
end % classdef



